
import netgen.gui
from ngsolve import *
from netgen.geom2d import SplineGeometry


geo = SplineGeometry()
geo.AddRectangle( (-1, -1), (1, 1),
                 bcs = ("bottom", "right", "top", "left"))
mesh = Mesh( geo.GenerateMesh(maxh=0.25))
Draw(mesh)
fes = H1(mesh, order=3, dirichlet="bottom")


u = fes.TrialFunction()  # symbolic object
v = fes.TestFunction()   # symbolic object
gfu = GridFunction(fes)  # solution
Draw(gfu)

dt = 0.001
t_end = 1
K = 1 #thermal conductivity
sources = 1 #soure term

#heat equation
a = BilinearForm(fes, symmetric=False)
a += SymbolicBFI(K * grad(u) * grad(v))
a.Assemble()

m = BilinearForm(fes)
m += SymbolicBFI(u * v)
m.Assemble()

mstar = m.mat.CreateMatrix()
mstar.AsVector().data = m.mat.AsVector() + dt * a.mat.AsVector()
invmstar = mstar.Inverse(freedofs=fes.FreeDofs())

f = LinearForm(fes)
f += SymbolicLFI(sources*v)
f.Assemble()

# This sets the function everywhere, including the boundary
gfu.Set(283.15, BND)
gfu.Set(293.15) #set initial condition
Redraw()
input("pause")

res = gfu.vec.CreateVector()
 
t = 0.0
#Implicit Euler scheme
while t <= t_end:

    res.data = m.mat * gfu.vec + dt * f.vec

    gfu.Set(283.15, BND)
    res.data += -mstar * gfu.vec
    gfu.vec.data += invmstar * res
    t = t+dt
    Redraw(blocking=True)
    


